﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Collections.ObjectModel;
using Infragistics.Windows.DataPresenter;

namespace DataErrorInfoShowErrorFromCode
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        MainViewModel vm = new MainViewModel();
        public MainWindow()
        {
            InitializeComponent();
            DataContext = vm;
        }

        private void XamDataGrid_InitializeRecord(object sender, Infragistics.Windows.DataPresenter.Events.InitializeRecordEventArgs e)
        {
            if (!string.IsNullOrEmpty(((e.Record as DataRecord).DataItem as IDataErrorInfo).Error))
                e.Record.Tag = ((e.Record as DataRecord).DataItem as IDataErrorInfo).Error;
            else
                e.Record.Tag = null;
        }

    }

    public class MainViewModel : ViewModelVase
    {
        private ObservableCollection<Data> data;

        public MainViewModel()
        {
            DataSource = new ObservableCollection<Data>();
            Random r = new Random();
            for (int i = 0; i < 10; i++)
            {
                DataSource.Add(new Data
                {
                    Name = i % 7 == 0 ? "i" + i : "Item " + i,
                    Size = i% 6 == 0 ? -2 : r.Next(20) + r.NextDouble(),
                });
            }
        }

        public ObservableCollection<Data> DataSource
        {
            get { return data; }
            set { data = value; }
        }

    }

    public class Data : ViewModelVase
    {
        private string name;
        private double size;

        public string Name
        {
            get { return name; }
            set
            {
                IsNameValid(value);
                name = value;
                OnPropertyChagned("Name");
            }

        }

        public double Size
        {
            get { return size; }
            set
            {
                IsSizeValid(value);

                size = value;
                OnPropertyChagned("Size");
            }
        }
        
        #region Validate
        private bool IsSizeValid(double value)
        {
            if (value < 0)
            {
                AddError("Size", "Size must be non-negative");
                return false;
            }
            else
            {
                RemoveError("Size", "Size must be non-negative");
                return true;
            }
        }
        private bool IsNameValid(string name)
        {
            if (string.IsNullOrEmpty(name) || name.Length < 3)
            {
                AddError("Name", "Name must be 3 chars or more");
                return false;
            }
            else
            {
                RemoveError("Name", "Name must be 3 chars or more");
                return true;
            }
        }
        #endregion


    }

    public class ViewModelVase : INotifyPropertyChanged, IDataErrorInfo
    {

        #region INotifyPropertyChanged Members
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChagned(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
        #endregion


        #region IDataErrorInfo members
        Dictionary<string, List<string>> errors = new Dictionary<string, List<string>>();

        public string Error
        {
            get
            {
                string rowError = "";
                foreach (var error in errors)
                {
                    rowError += error.Key + ":\n";
                    foreach (string err in error.Value)
                    {
                        rowError += "\t" + err + "\n";
                    }
                }                
                return rowError; 
                
            }
        }

        public string this[string columnName]
        {
            get
            {
                return (!errors.ContainsKey(columnName) ? null :
                      String.Join(Environment.NewLine, errors[columnName]));
            }
        }

        //Add errors
        public void AddError(string prop, string err)
        {
            if (!errors.ContainsKey(prop))
                errors.Add(prop, new List<string>());
            if (!errors[prop].Contains(err))
                errors[prop].Add(err);
        }
        //Remove errors
        public void RemoveError(string prop, string err)
        {
            if (errors.ContainsKey(prop) &&
              errors[prop].Contains(err))
            {
                errors[prop].Remove(err);
                if (errors[prop].Count == 0) errors.Remove(prop);
            }
        }
        #endregion
    }
}
